﻿/*
 * Arduino.cs - Arduino/firmata library for Visual C# .NET
 * Copyright (C) 2009 Tim Farley http://www.imagitronics.org/projects/firmatanet/
 * 
 * Modified by Andrei Skoogarev
 */

using System;
using System.IO.Ports;
using System.Linq;
using System.Threading;

namespace Firmata.NET {
	/**
	 * Together with the Firmata 2 firmware (an Arduino sketch uploaded to the
	 * Arduino board), this class allows you to control the Arduino board from
	 * Processing: reading from and writing to the digital pins and reading the
	 * analog inputs.
	 */
	public class ShieldIO {
		#region Constants
		public const Byte MODE_INPUT = 0x00;
		public const Byte MODE_OUTPUT = 0x01;
		public const Byte MODE_ANALOG = 0x02;
		public const Byte MODE_PWM = 0x03;
		public const Byte MODE_SERVO = 0x04;
		public const Byte MODE_SHIFT = 0x05;
		public const Byte MODE_I2C = 0x06;

		public const Byte LOW = 0;
		public const Byte HIGH = 1;

		private const Int32 MAX_DATA_BYTES = 32;

		private const Byte DIGITAL_MESSAGE = 0x90; // send data for a digital port
		private const Byte ANALOG_MESSAGE = 0xE0; // send data for an analog pin (or PWM)
		private const Byte REPORT_ANALOG = 0xC0; // enable analog input by pin #
		private const Byte REPORT_DIGITAL = 0xD0; // enable digital input by port
		private const Byte SET_PIN_MODE = 0xF4; // set a pin to INPUT/OUTPUT/PWM/etc
		private const Byte REPORT_VERSION = 0xF9; // report firmware version
		private const Byte SYSTEM_RESET = 0xFF; // reset from MIDI
		private const Byte START_SYSEX = 0xF0; // start a MIDI SysEx message
		private const Byte END_SYSEX = 0xF7; // end a MIDI SysEx message
		#endregion

		#region Variables
		private SerialPort _serialPort;
		private Int32 _delayMs;

		private Int32 _waitForData;
		private Int32 _executeMultiByteCommand;
		private Int32 _multiByteChannel;
		private Int32[] _storedInputData = new int[MAX_DATA_BYTES];
		private Boolean _parsingSysex;
		private Int32 _sysexBytesRead;

		private volatile Int32[] _digitalOutputData = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		private volatile Int32[] _digitalInputData = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		private volatile Int32[] _analogInputData = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

		private Int32 _majorVersion;
		private Int32 _minorVersion;
		private Thread _readThread;
		private Object _locker = new Object();
		#endregion

		#region Ctor
		/// <summary>
		/// 
		/// </summary>
		/// <param name="serialPortName">String specifying the name of the serial port. eg COM4</param>
		/// <param name="baudRate">The baud rate of the communication. Default 115200</param>
		/// <param name="autoStart">Determines whether the serial port should be opened automatically.
		///                     use the Open() method to open the connection manually.</param>
		/// <param name="delay">Time delay that may be required to allow some arduino models
		///                     to reboot after opening a serial connection. The delay will only activate
		///                     when autoStart is true.</param>
		public ShieldIO(String serialPortName, Int32 baudRate, Boolean autoStart, Int32 delayMs) {
			this._serialPort = new SerialPort(serialPortName, baudRate);
			this._serialPort.DataBits = 8;
			this._serialPort.Parity = Parity.None;
			this._serialPort.StopBits = StopBits.One;

			if (autoStart) {
				this._delayMs = delayMs;
				this.Open();
			}
		}
		#endregion

		#region Public
		/// <summary>
		/// Opens the serial port connection, should it be required. By default the port is
		/// opened when the object is first created.
		/// </summary>
		public void Open() {
			this._serialPort.Open();
			Thread.Sleep(this._delayMs);

			if (this._readThread == null) {
				this._readThread = new Thread(this._processInput);
				this._readThread.Start();
			}
		}

		/// <summary>
		/// Closes the serial port.
		/// </summary>
		public void Close() {
			_readThread.Join(500);
			_readThread = null;
			this._serialPort.Close();
		}

		/// <summary>
		/// Returns the last known state of the digital pin.
		/// </summary>
		/// <param name="pin">The arduino digital input pin.</param>
		/// <returns>Arduino.HIGH or Arduino.LOW</returns>
		public Int32 DigitalRead(Int32 pin) {
			return (this._digitalInputData[pin >> 3] >> (pin & 0x07)) & 0x01;
		}

		/// <summary>
		/// Returns the last known state of the analog pin.
		/// </summary>
		/// <param name="pin">The arduino analog input pin.</param>
		/// <returns>A value representing the analog value between 0 (0V) and 1023 (5V).</returns>
		public Int32 AnalogRead(Int32 pin) {
			return this._analogInputData[pin];
		}

		/// <summary>
		/// Sets the mode of the specified pin (INPUT or OUTPUT).
		/// </summary>
		/// <param name="pin">The arduino pin.</param>
		/// <param name="mode">Mode Arduino.INPUT or Arduino.OUTPUT.</param>
		public void SetPinMode(Int32 pin, Int32 mode) {
			Byte[] message = new Byte[3];
			message[0] = (Byte) (SET_PIN_MODE);
			message[1] = (Byte) (pin);
			message[2] = (Byte) (mode);
			this._serialPort.Write(message, 0, 3);
			message = null;
		}

		/// <summary>
		/// Write to a digital pin that has been toggled to output mode with pinMode() method.
		/// </summary>
		/// <param name="pin">The digital pin to write to.</param>
		/// <param name="value">Value either Arduino.LOW or Arduino.HIGH.</param>
		public void DigitalWrite(Int32 pin, Int32 value) {
			Int32 portNumber = (pin >> 3) & 0x0F;
			Byte[] message = new Byte[3];

			if (value == 0)
				this._digitalOutputData[portNumber] &= ~(1 << (pin & 0x07));
			else
				this._digitalOutputData[portNumber] |= (1 << (pin & 0x07));

			message[0] = (Byte) (DIGITAL_MESSAGE | portNumber);
			message[1] = (Byte) (this._digitalOutputData[portNumber] & 0x7F);
			message[2] = (Byte) (this._digitalOutputData[portNumber] >> 7);
			this._serialPort.Write(message, 0, 3);
		}

		/// <summary>
		/// Write to an analog pin using Pulse-width modulation (PWM).
		/// </summary>
		/// <param name="pin">Analog output pin.</param>
		/// <param name="value">PWM frequency from 0 (always off) to 255 (always on).</param>
		public void AnalogWrite(Int32 pin, Int32 value) {
			Byte[] message = new Byte[3];
			message[0] = (Byte) (ANALOG_MESSAGE | (pin & 0x0F));
			message[1] = (Byte) (value & 0x7F);
			message[2] = (Byte) (value >> 7);
			this._serialPort.Write(message, 0, 3);
		}
		#endregion

		#region Private
		private void _processInput() {
			while (this._serialPort.IsOpen) {
				if (this._serialPort.BytesToRead > 0) {
					lock (this) {
						Int32 inputData = this._serialPort.ReadByte();
						Int32 command;

						if (this._parsingSysex) {
							if (inputData == END_SYSEX) {
								this._parsingSysex = false;
								//processSysexMessage();
							}
							else {
								this._storedInputData[this._sysexBytesRead] = inputData;
								this._sysexBytesRead++;
							}
						}
						else if (this._waitForData > 0 && inputData < 128) {
							this._waitForData--;
							this._storedInputData[_waitForData] = inputData;

							if (this._executeMultiByteCommand != 0 && this._waitForData == 0) {
								// We got everything
								switch (this._executeMultiByteCommand) {
									case DIGITAL_MESSAGE:
										this._digitalInputData[this._multiByteChannel] = (this._storedInputData[0] << 7) + this._storedInputData[1];
										break;
									case ANALOG_MESSAGE:
										this._analogInputData[this._multiByteChannel] = (this._storedInputData[0] << 7) + this._storedInputData[1];
										break;
									case REPORT_VERSION:
										this._majorVersion = this._storedInputData[1];
										this._minorVersion = this._storedInputData[0];
										break;
								}
							}
						}
						else {
							if (inputData < 0xF0) {
								command = inputData & 0xF0;
								this._multiByteChannel = inputData & 0x0F;
							}
							else {
								command = inputData;
								// commands in the 0xF* range don't use channel data
							}
							switch (command) {
								case DIGITAL_MESSAGE:

								case ANALOG_MESSAGE:
								case REPORT_VERSION:
									this._waitForData = 2;
									this._executeMultiByteCommand = command;
									break;
							}
						}
					}
				}
			}
		}
		#endregion
	}
}