﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;
using Firmata.NET;

namespace OdroidShield {
	class Program {
		#region Constants
		private const String PORT_NAME_LINUX = "/dev/ttyACM99";
		private const Int32 PORT_BAUD_RATE = 57600;

		private const Byte START_SYSEX = 0xF0; // start a MIDI Sysex message
		private const Byte END_SYSEX = 0xF7; // end a MIDI Sysex message
		private const Byte REPORT_FIRMWARE = 0x79; // report name and version of the firmware
		#endregion

		#region Variables
		private SerialPort _serialPort1;
		#endregion

		#region Main
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			new Program()._runFirmwareVersionRead();
			new Program()._runPinTest();
		}
		#endregion

		#region Utils
		private void _runPinTest() {
			try {
				Console.WriteLine("Start pin test");
				ShieldIO arduino = new ShieldIO(PORT_NAME_LINUX, PORT_BAUD_RATE, true, 0);
				
				// D3 и D5 выведены на трехпиновые разъемы для серв
				arduino.SetPinMode(3, ShieldIO.MODE_SERVO);
				arduino.SetPinMode(5, ShieldIO.MODE_SERVO);
				
				// D13 синий светодиод
				arduino.SetPinMode(13, ShieldIO.MODE_OUTPUT);

				// Ставим сервы по центру
				arduino.AnalogWrite(3, 1500);
				arduino.AnalogWrite(5, 1500);

				// Моргаем светодиодом, двигаем сервы
				for (Int32 i = 0; i < 10; i++) {
					arduino.AnalogWrite(3, 1000 + i * 100);
					arduino.AnalogWrite(5, 1000 + i * 100);

					Console.Write("[" + (i + 1) + "] D13=1");
					arduino.DigitalWrite(13, ShieldIO.HIGH);
					Thread.Sleep(500);
					Console.WriteLine(" 0");
					arduino.DigitalWrite(13, ShieldIO.LOW);
					Thread.Sleep(500);
				}
				
				arduino.Close();
			}
			catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}
		}

		private void _runFirmwareVersionRead() {
			try {
				this._portOpen();
				this._readFirmwareVersion();
				this._portClose();
			}
			catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}
		}

		private void _portOpen() {
			this._serialPort1 = new SerialPort();
			this._serialPort1.PortName = PORT_NAME_LINUX;
			this._serialPort1.BaudRate = PORT_BAUD_RATE;
			
			//this._serialPort1.BaudRate = 9600;
			this._serialPort1.Open();

			// Читаем всякий мусор перед тем как посылать
			while (this._serialPort1.BytesToRead > 0) { this._serialPort1.ReadByte(); }
		}

		private void _portClose() {
			if (this._serialPort1.IsOpen) { this._serialPort1.Close(); }
		}

		/// <summary>
		/// Read firmata name & version
		/// </summary>
		/// <param name="buffer"></param>
		/// <param name="readBytes"></param>
		/// <param name="str"></param>
		/// <returns></returns>
		private void _readFirmwareVersion() {
			Console.WriteLine("Read firmware version");
			Byte[] buf = new Byte[] { START_SYSEX, REPORT_FIRMWARE, END_SYSEX };
			//Byte[] buf = new Byte[] { (Byte) 'V', 0x0A };
			this._serialPort1.Write(buf, 0, buf.Length);

			Int32 counter = 0;
			while (true) {
				Int32 readBytes = this._serialPort1.BytesToRead;
				if (readBytes <= 0) {
					counter++;
					if (counter >= 500) { Console.WriteLine("Error: no answer"); break; }
					Thread.Sleep(10);
					continue;
				}
				
				Console.Write(counter + " " + readBytes + ": ");
				Boolean showName = false;
				for (Int32 i = 0; i < readBytes; i++) {
					Byte bt = (Byte) this._serialPort1.ReadByte();
					if (bt == 0) { continue; }

					if (bt == START_SYSEX) { showName = true; continue; }
					if (bt == END_SYSEX) { break; }
					if (showName) {
						if (bt < 32 || bt > 127)
							Console.Write("[0x" + bt.ToString("X2") + "]");
						else
							Console.Write((Char) bt);
					}
				}
				Console.WriteLine("");
				break;
			}	
		}		
		#endregion
	}
}